package main
/**
* 房屋价值分析报告
*/
import (
	"fmt"
	"strings"
	"net/url"
	"net/http"
	"io/ioutil"
	"encoding/json"
)

//获取token使用的appKey和appSecret
var appKey = "appKey";
var appSecret = "appSecret";

//房屋价值分析报告--预判
func PriceReportPre(param PriceReportParam, token string) {
	//请求参数
	var params = url.Values{
		"identifier": {param.Identifier},
		"standPriceAssessBuildArea": {param.StandPriceAssessBuildArea},
		"standDetailedAddress": {param.StandDetailedAddress},
		"standCity": {param.StandCity},
		"cityGbCode": {param.CityGbCode},
		"standDistrictAndCounty": {param.StandDistrictAndCounty},
		"standResblock": {param.StandResblock},
		"standBuilding": {param.StandBuilding},
		"standUnit": {param.StandUnit},
		"standHouse": {param.StandHouse},
	}
	//请求地址
	req, _ := http.NewRequest(http.MethodPost,
		"https://gw-open.ke.com/api/assessPriceReportPrejudge",
		strings.NewReader(params.Encode()))
	req.Header.Add("Content-Type","application/x-www-form-urlencoded")
	req.Header.Add("access_token",token)
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
        fmt.Println("调用房屋价值分析报告--预判出错")
    }
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	fmt.Println(string(body))
}

//房屋价值分析报告--申请
func PriceReportApply(param PriceReportParam, token string) {
	//请求参数
	var params = url.Values{
		"assessType": {param.AssessType},
		"identifier": {param.Identifier},
		"standPriceAssessBuildArea": {param.StandPriceAssessBuildArea},
		"standDetailedAddress": {param.StandDetailedAddress},
		"standCity": {param.StandCity},
		"cityGbCode": {param.CityGbCode},
		"standDistrictAndCounty": {param.StandDistrictAndCounty},
		"standResblock": {param.StandResblock},
		"standBuilding": {param.StandBuilding},
		"standUnit": {param.StandUnit},
		"standHouse": {param.StandHouse},
	}
	//请求地址
	req, _ := http.NewRequest(http.MethodPost,
		"https://gw-open.ke.com/api/assessPriceReportApply",
		strings.NewReader(params.Encode()))
	req.Header.Add("Content-Type","application/x-www-form-urlencoded")
	req.Header.Add("access_token",token)
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
        fmt.Println("调用房屋价值分析报告--申请出错")
    }
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	fmt.Println(string(body))
}

//房屋价值分析报告--进度
func PriceReportProgress(identifier string, token string) {
	//请求参数
	var params = url.Values{
		"identifier": {identifier},
	}
	//请求地址
	req, _ := http.NewRequest(http.MethodPost,
		"https://gw-open.ke.com/api/assessPriceReportProgress",
		strings.NewReader(params.Encode()))
	//设置请求header
	req.Header.Add("Content-Type","application/x-www-form-urlencoded")
	req.Header.Add("access_token",token)
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
        fmt.Println("调用房屋价值分析报告--进度出错")
    }
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	fmt.Println(string(body))
}

//房屋价值分析报告--获取
func PriceReport(identifier string, token string) {
	//请求参数
	var params = url.Values{
		"identifier": {identifier},
	}
	//请求地址
	req, _ := http.NewRequest(http.MethodPost,
		"https://gw-open.ke.com/api/assessPriceReport",
		strings.NewReader(params.Encode()))
	//设置请求header
	req.Header.Add("Content-Type","application/x-www-form-urlencoded")
	req.Header.Add("access_token",token)
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
        fmt.Println("调用房屋价值分析报告--获取出错")
    }
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	fmt.Println(string(body))
}

//获取token
func GetToken() string {
	//请求参数
	params := url.Values{
		"grant_type": {"client_credentials"},
		"client_id": {appKey},
		"client_secret": {appSecret},
	}
	//请求地址
	req, _ := http.NewRequest(http.MethodPost,
		"https://gw-open.ke.com/oauth/token",
		strings.NewReader(params.Encode()))
	req.Header.Add("Content-Type","application/x-www-form-urlencoded")
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
        return ""
    }
	defer resp.Body.Close()
	body, _ := ioutil.ReadAll(resp.Body)
	var res Response
	json.Unmarshal([]byte(body), &res)
	return res.Data.AccessToken
}

//返回的token数据结构
type Token struct {
	AccessToken string `json:"access_token"`
	RefreshToken string `json:"refresh_token"`
	TokenType string `json:"token_type"`
	ExpiresIn string `json:"expires_in"`
	Scope string `json:"scope"`
}
type Response struct {
	Code int32 `json:"code"`
	Msg string `json:"msg"`
	Data Token `json:"data"`
}

//房屋价值分析报告参数
type PriceReportParam struct{
	AssessType string
	Identifier string
	StandPriceAssessBuildArea string
	StandDetailedAddress string
	StandCity string
	CityGbCode string
	StandDistrictAndCounty string
	StandResblock string
	StandBuilding string
	StandUnit string
	StandHouse string
}

func main() {
	var token string
	token = GetToken()
	//房e估
	fun(token)
}

//房屋价值分析报告
func fun(token string) {
	var priceReportParam PriceReportParam
	priceReportParam.AssessType = "SECOND_HAND_REPORT_ASSESS"
	priceReportParam.Identifier = "1djiuduf"
	priceReportParam.StandPriceAssessBuildArea = "66.3"
	priceReportParam.StandDetailedAddress = "北京市昌平区回龙观镇龙乡小区东区14号楼1单元601"
	priceReportParam.StandCity = "北京市"
	priceReportParam.CityGbCode = "110000"
	priceReportParam.StandDistrictAndCounty = "昌平区"
	priceReportParam.StandResblock = "龙乡小区东区"
	priceReportParam.StandBuilding = "14号楼"
	priceReportParam.StandUnit = "1单元"
	priceReportParam.StandHouse = "601"
	PriceReportPre(priceReportParam, token)
	PriceReportApply(priceReportParam, token)
	PriceReportProgress(priceReportParam.Identifier, token)
	PriceReport(priceReportParam.Identifier, token)
}
