package com.ke.open.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.CharEncoding;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 实勘验真
 * <p>
 * 服务依赖于Apache的HttpClient以及FastJSON, 以下是pom依赖
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpcore</artifactId>
 * <version>4.4.10</version>
 * </dependency>
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpclient</artifactId>
 * <version>4.5.6</version>
 * </dependency>
 * <dependency>
 * <groupId>com.alibaba</groupId>
 * <artifactId>fastjson</artifactId>
 * <version>1.2.62</version>
 * </dependency>
 */
public class ShotVerify {

    private static final String AppKey = "appKey";

    private static final String AppSecret = "appSecret";

    /**
     * 下单前判断
     */
    public void shotVerify(ShotVerifyParam param, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/images-distinguish/order/checkPrivilege");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        JSONObject jsonParam = JSON.parseObject(JSON.toJSONString(param), JSONObject.class);
        for (Map.Entry<String, Object> entry : jsonParam.entrySet()) {
            nvps.add(new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue())));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("实勘验真返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("实勘验真调用失败: " + httpResponse);
        }
    }

    /**
     * 获取token
     */
    private String getToken() throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/oauth/token");
        //请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("grant_type", "client_credentials"));
        nvps.add(new BasicNameValuePair("client_id", AppKey));
        nvps.add(new BasicNameValuePair("client_secret", AppSecret));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //请求参数类型, 使用urlencoded方式进行编码
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        //检查返回结果的返回状态是否是http200,
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            EntityUtils.consume(httpEntity);
            return Optional.ofNullable(jsonObject)
                    .map(data -> data.get("data"))
                    .map(data -> JSONObject.parseObject(JSON.toJSONString(data), JSONObject.class))
                    .map(data -> data.get("access_token"))
                    .map(String::valueOf)
                    .orElse("");
        }
        return "";
    }

    private static class ShotVerifyParam {
        String cityName;
        String districtName;
        String resblockName;
        String buildingName;
        String unitName;
        String houseName;

        public String getCityName() {
            return cityName;
        }

        public void setCityName(String cityName) {
            this.cityName = cityName;
        }

        public String getDistrictName() {
            return districtName;
        }

        public void setDistrictName(String districtName) {
            this.districtName = districtName;
        }

        public String getResblockName() {
            return resblockName;
        }

        public void setResblockName(String resblockName) {
            this.resblockName = resblockName;
        }

        public String getBuildingName() {
            return buildingName;
        }

        public void setBuildingName(String buildingName) {
            this.buildingName = buildingName;
        }

        public String getUnitName() {
            return unitName;
        }

        public void setUnitName(String unitName) {
            this.unitName = unitName;
        }

        public String getHouseName() {
            return houseName;
        }

        public void setHouseName(String houseName) {
            this.houseName = houseName;
        }
    }

    public static void main(String[] args) throws IOException {
        ShotVerify shotVerify = new ShotVerify();
        String token = shotVerify.getToken();
        ShotVerifyParam param = new ShotVerifyParam();
        param.setCityName("北京市");
        param.setDistrictName("海淀区");
        param.setResblockName("龙腾苑二区");
        param.setBuildingName("13号楼");
        param.setUnitName("5单元");
        param.setHouseName("101");
        shotVerify.shotVerify(param, token);
    }
}