package com.ke.open.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.CharEncoding;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 成交案例库
 * <p>
 * 服务依赖于Apache的HttpClient以及FastJSON, 以下是pom依赖
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpcore</artifactId>
 * <version>4.4.10</version>
 * </dependency>
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpclient</artifactId>
 * <version>4.5.6</version>
 * </dependency>
 * <dependency>
 * <groupId>com.alibaba</groupId>
 * <artifactId>fastjson</artifactId>
 * <version>1.2.62</version>
 * </dependency>
 */
public class DealExample {

    private static final String AppKey = "appKey";

    private static final String AppSecret = "appSecret";

    /**
     * 成交案例库接口
     */
    public void dealExample(DealExampleParam param, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/assessTransactionCase");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        JSONObject jsonParam = JSON.parseObject(JSON.toJSONString(param), JSONObject.class);
        for (Map.Entry<String, Object> entry : jsonParam.entrySet()) {
            nvps.add(new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue())));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("成交案例库返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("成交案例库调用失败: " + httpResponse);
        }
    }

    /**
     * 获取token
     */
    private String getToken() throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/oauth/token");
        //请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("grant_type", "client_credentials"));
        nvps.add(new BasicNameValuePair("client_id", AppKey));
        nvps.add(new BasicNameValuePair("client_secret", AppSecret));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //请求参数类型, 使用urlencoded方式进行编码
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        //检查返回结果的返回状态是否是http200,
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            EntityUtils.consume(httpEntity);
            return Optional.ofNullable(jsonObject)
                    .map(data -> data.get("data"))
                    .map(data -> JSONObject.parseObject(JSON.toJSONString(data), JSONObject.class))
                    .map(data -> data.get("access_token"))
                    .map(String::valueOf)
                    .orElse("");
        }
        return "";
    }

    private static class DealExampleParam {
        String standCity;
        String standPriceAssessBuildArea;
        String standDetailedAddress;
        String timePeriod;

        public String getStandCity() {
            return standCity;
        }

        public void setStandCity(String standCity) {
            this.standCity = standCity;
        }

        public String getStandPriceAssessBuildArea() {
            return standPriceAssessBuildArea;
        }

        public void setStandPriceAssessBuildArea(String standPriceAssessBuildArea) {
            this.standPriceAssessBuildArea = standPriceAssessBuildArea;
        }

        public String getStandDetailedAddress() {
            return standDetailedAddress;
        }

        public void setStandDetailedAddress(String standDetailedAddress) {
            this.standDetailedAddress = standDetailedAddress;
        }

        public String getTimePeriod() {
            return timePeriod;
        }

        public void setTimePeriod(String timePeriod) {
            this.timePeriod = timePeriod;
        }
    }

    public static void main(String[] args) throws IOException {
        DealExample dealExample = new DealExample();
        String token = dealExample.getToken();
        DealExample.DealExampleParam param = new DealExampleParam();
        param.setStandCity("重庆市");
        param.setStandDetailedAddress("重庆市龙湖春森彼岸四期");
        param.setTimePeriod("LAST_THREE_MONTH");
        param.setStandPriceAssessBuildArea("99.01");
        dealExample.dealExample(param, token);
    }
}