package com.ke.open.service;

/**
 * @author fengyadong001
 * @date 2020/5/18 4:01 PM
 * @description
 */

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.CharEncoding;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 房屋价值分析报告
 * <p>
 * 服务依赖于Apache的HttpClient以及FastJSON, 以下是pom依赖
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpcore</artifactId>
 * <version>4.4.10</version>
 * </dependency>
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpclient</artifactId>
 * <version>4.5.6</version>
 * </dependency>
 * <dependency>
 * <groupId>com.alibaba</groupId>
 * <artifactId>fastjson</artifactId>
 * <version>1.2.62</version>
 * </dependency>
 */
public class PriceReport {

    private static final String AppKey = "appKey";

    private static final String AppSecret = "appSecret";

    /**
     * 房屋价值分析报告--预判
     */
    public void priceReportPre(PriceReportParam param, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/assessPriceReportPrejudge");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        JSONObject jsonParam = JSON.parseObject(JSON.toJSONString(param), JSONObject.class);
        for (Map.Entry<String, Object> entry : jsonParam.entrySet()) {
            nvps.add(new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue())));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("房屋价值分析报告--预判返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("房屋价值分析报告--预判调用失败: " + httpResponse);
        }
    }


    /**
     * 房屋价值分析报告--申请
     */
    public void priceReportApply(PriceReportParam param, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/assessPriceReportApply");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        JSONObject jsonParam = JSON.parseObject(JSON.toJSONString(param), JSONObject.class);
        for (Map.Entry<String, Object> entry : jsonParam.entrySet()) {
            nvps.add(new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue())));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("房屋价值分析报告--申请返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("房屋价值分析报告--申请调用失败: " + httpResponse);
        }
    }


    /**
     * 房屋价值分析报告--进度
     */
    public void priceReportProgress(String identifier, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/assessPriceReportProgress");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("identifier", identifier));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("房屋价值分析报告--进度返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("房屋价值分析报告--进度调用失败: " + httpResponse);
        }
    }

    /**
     * 房屋价值分析报告--获取
     */
    public void priceReport(String identifier, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/assessPriceReport");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("identifier", identifier));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("房屋价值分析报告--获取返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("房屋价值分析报告--获取调用失败: " + httpResponse);
        }
    }

    /**
     * 获取token
     */
    private String getToken() throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/oauth/token");
        //请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("grant_type", "client_credentials"));
        nvps.add(new BasicNameValuePair("client_id", AppKey));
        nvps.add(new BasicNameValuePair("client_secret", AppSecret));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //请求参数类型, 使用urlencoded方式进行编码
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        //检查返回结果的返回状态是否是http200,
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            EntityUtils.consume(httpEntity);
            return Optional.ofNullable(jsonObject)
                    .map(data -> data.get("data"))
                    .map(data -> JSONObject.parseObject(JSON.toJSONString(data), JSONObject.class))
                    .map(data -> data.get("access_token"))
                    .map(String::valueOf)
                    .orElse("");
        }
        return "";
    }


    private static class PriceReportParam{
        String assessType;
        String identifier;
        Float standPriceAssessBuildArea;
        String standDetailedAddress;
        String standCity;
        Integer cityGbCode;
        String standDistrictAndCounty;
        String standResblock;
        String standBuilding;
        String standUnit;
        String standHouse;

        public String getAssessType() {
            return assessType;
        }

        public void setAssessType(String assessType) {
            this.assessType = assessType;
        }

        public String getIdentifier() {
            return identifier;
        }

        public void setIdentifier(String identifier) {
            this.identifier = identifier;
        }

        public Float getStandPriceAssessBuildArea() {
            return standPriceAssessBuildArea;
        }

        public void setStandPriceAssessBuildArea(Float standPriceAssessBuildArea) {
            this.standPriceAssessBuildArea = standPriceAssessBuildArea;
        }

        public String getStandDetailedAddress() {
            return standDetailedAddress;
        }

        public void setStandDetailedAddress(String standDetailedAddress) {
            this.standDetailedAddress = standDetailedAddress;
        }

        public String getStandCity() {
            return standCity;
        }

        public void setStandCity(String standCity) {
            this.standCity = standCity;
        }

        public Integer getCityGbCode() {
            return cityGbCode;
        }

        public void setCityGbCode(Integer cityGbCode) {
            this.cityGbCode = cityGbCode;
        }

        public String getStandDistrictAndCounty() {
            return standDistrictAndCounty;
        }

        public void setStandDistrictAndCounty(String standDistrictAndCounty) {
            this.standDistrictAndCounty = standDistrictAndCounty;
        }

        public String getStandResblock() {
            return standResblock;
        }

        public void setStandResblock(String standResblock) {
            this.standResblock = standResblock;
        }

        public String getStandBuilding() {
            return standBuilding;
        }

        public void setStandBuilding(String standBuilding) {
            this.standBuilding = standBuilding;
        }

        public String getStandUnit() {
            return standUnit;
        }

        public void setStandUnit(String standUnit) {
            this.standUnit = standUnit;
        }

        public String getStandHouse() {
            return standHouse;
        }

        public void setStandHouse(String standHouse) {
            this.standHouse = standHouse;
        }
    }

    public static void main(String[] args) throws IOException {
        PriceReport priceReport = new PriceReport();
        String token = priceReport.getToken();
        PriceReportParam param = new PriceReportParam();
        param.setAssessType("SECOND_HAND_REPORT_ASSESS");
        param.setIdentifier("1djiuduf");
        param.setStandPriceAssessBuildArea(66.3f);
        param.setStandDetailedAddress("北京市昌平区回龙观镇龙乡小区东区14号楼1单元601");
        param.setStandCity("北京市");
        param.setCityGbCode(110000);
        param.setStandDistrictAndCounty("昌平区");
        param.setStandResblock("龙乡小区东区");
        param.setStandBuilding("14号楼");
        param.setStandUnit("1单元");
        param.setStandHouse("601");
        priceReport.priceReportPre(param, token);
        priceReport.priceReportApply(param, token);
        priceReport.priceReportProgress(param.getIdentifier(), token);
        priceReport.priceReport(param.getIdentifier(), token);
    }
}