package com.ke.open.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.CharEncoding;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import sun.misc.BASE64Encoder;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 房本识别
 * <p>
 * 服务依赖于Apache的HttpClient以及FastJSON, 以下是pom依赖
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpcore</artifactId>
 * <version>4.4.10</version>
 * </dependency>
 * <dependency>
 * <groupId>org.apache.httpcomponents</groupId>
 * <artifactId>httpclient</artifactId>
 * <version>4.5.6</version>
 * </dependency>
 * <dependency>
 * <groupId>com.alibaba</groupId>
 * <artifactId>fastjson</artifactId>
 * <version>1.2.62</version>
 * </dependency>
 */
public class HouseCert {

    private static final String AppKey = "appKey";

    private static final String AppSecret = "appSecret";

    /**
     * 房本识别
     */
    public void houseCert(HouseCertParam param, String token) throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/api/ocr/houseCert");
        //设置请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        JSONObject jsonParam = JSON.parseObject(JSON.toJSONString(param), JSONObject.class);
        for (Map.Entry<String, Object> entry : jsonParam.entrySet()) {
            nvps.add(new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue())));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //设置请求header, 设置body的编码方式以及access_token
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        httpPost.setHeader("access_token", token);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            System.out.println("房产证识别返回数据: " + jsonObject);
            EntityUtils.consume(httpEntity);
        } else {
            System.out.println("房产证识别调用失败: " + httpResponse);
        }
    }

    /**
     * 获取token
     */
    private String getToken() throws IOException {
        //设置请求的地址
        HttpPost httpPost = new HttpPost("https://gw-open.ke.com/oauth/token");
        //请求参数
        List<NameValuePair> nvps = new ArrayList<>();
        nvps.add(new BasicNameValuePair("grant_type", "client_credentials"));
        nvps.add(new BasicNameValuePair("client_id", AppKey));
        nvps.add(new BasicNameValuePair("client_secret", AppSecret));
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, CharEncoding.UTF_8));
        //请求参数类型, 使用urlencoded方式进行编码
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded");
        CloseableHttpClient httpClient = HttpClients.createDefault();
        //获取响应结果
        CloseableHttpResponse httpResponse = httpClient.execute(httpPost);
        //检查返回结果的返回状态是否是http200,
        if (httpResponse.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            //获取响应体, 解析数据并返回返回
            HttpEntity httpEntity = httpResponse.getEntity();
            JSONObject jsonObject = JSON.parseObject(EntityUtils.toString(httpEntity), JSONObject.class);
            EntityUtils.consume(httpEntity);
            return Optional.ofNullable(jsonObject)
                    .map(data -> data.get("data"))
                    .map(data -> JSONObject.parseObject(JSON.toJSONString(data), JSONObject.class))
                    .map(data -> data.get("access_token"))
                    .map(String::valueOf)
                    .orElse("");
        }
        return "";
    }

    /**
     * 图片转base64
     * */
    private String convertFileToBase64(String imgPath) {
        byte[] data = null;
        // 读取图片字节数组
        try {
            InputStream in = new FileInputStream(imgPath);
            data = new byte[in.available()];
            in.read(data);
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 对字节数组进行Base64编码，得到Base64编码的字符串
        return new BASE64Encoder().encode(data);
    }

    //房本识别参数
    private static class HouseCertParam{
        String imgUrl;

        String imgBase64;

        public String getImgUrl() {
            return imgUrl;
        }

        public void setImgUrl(String imgUrl) {
            this.imgUrl = imgUrl;
        }

        public String getImgBase64() {
            return imgBase64;
        }

        public void setImgBase64(String imgBase64) {
            this.imgBase64 = imgBase64;
        }
    }

    public static void main(String[] args) throws IOException {
        HouseCert houseCert = new HouseCert();
        String token = houseCert.getToken();
        HouseCertParam param = new HouseCertParam();
        param.setImgUrl("https://s1.ljcdn.com/transaction-public-static/tradeBrain/static/fangben-1-detect.png");
        houseCert.houseCert(param, token);
        param.setImgUrl(null);
        //房本图片路径
        String path = "/Users/fengyadong/fangben-1-detect.png";
        String base64 = houseCert.convertFileToBase64(path);
        param.setImgBase64(base64);
        houseCert.houseCert(param, token);
    }
}