from . import config

from urllib import parse

import requests
import json

TOKEN_PATH = '/oauth/token'


def get_token(client_id, client_secret):
    """获取token.

    param client_id: 应用ak

    param client_secret: 应用sk
    """

    url = config.GW_URL + TOKEN_PATH

    headers = {
        'Content-Type': config.CONTENT_FORM_URLENCODED
    }
    data = {
        'grant_type': 'client_credentials',
        'client_id': client_id,
        'client_secret': client_secret
    }

    response = requests.post(url, headers=headers, data=parse.urlencode(data))
    if response.status_code == 200:
        result = json.loads(response.text)
        if result['code'] == 0:
            return result['data']
        else:
            raise RuntimeError("failed to get token: " + result['msg'])
    else:
        raise RuntimeError("failed to get token, http stauts code is %d" % (response.status_code))
    



def refresh_token(client_id, client_secret, refresh_token):
    """刷新token.

    param client_id: 应用ak

    param client_secret: 应用sk

    param refresh_token: 刷新时认证身份的token

    """

    headers = {
        'Content-Type': config.CONTENT_FORM_URLENCODED
    }
    data = {
        'grant_type': 'refresh_token',
        'refresh_token': refresh_token,
        'client_id': client_id,
        'client_secret': client_secret
    }
    url = config.GW_URL + TOKEN_PATH

    response = requests.post(url, headers=headers, data=parse.urlencode(data))
    if response.status_code == 200:
        result = json.loads(response.text)
        if result['code'] == 0:
            return result['data']
        else:
            raise RuntimeError("failed to get token: " + result['msg'])
    else:
        raise RuntimeError("failed to get token, http stauts code is %d" % (response.status_code))



__all__ = ['get_token', 'refresh_token']
