from ..config import *
from urllib import parse
from ..auth import (
    get_token,
    refresh_token
)
from datetime import datetime

import requests
import json

class _SvcCall:
    """
    内部服务api调用实现类, 各个服务继承该类
    """
    
    def __init__(self, **kw):
        """
        参数 client_id : ak;
        参数 client_secret: sk;
        参数 token: access token

        使用时，可以传 client_id, client_secret 或者只传 token，二者区别是前者支持自动刷新token
        """
        (client_id, client_secret, token) = (None, None, None)
        if 'client_id' in kw: 
            client_id =  kw['client_id']
        if 'client_secret' in kw:
            client_secret = kw['client_secret']
        if 'token' in kw:
            token = kw['token']

        if token != None:
            self.token = token
        else:
            if client_id != None and client_secret != None:
                self.refreshable = True
                self.client_id = client_id
                self.client_secret = client_secret

                token_data = get_token(client_id, client_secret)
                self.token = token_data[ACCESS_TOKEN]
                self.refresh_token = token_data[REFRESH_TOKEN]
                self.last_token_time = datetime.now() # 获取token的时间
                self.token_expire_seconds = token_data[EXPIRES_IN]
            else:
                raise ValueError('初始化错误，参数有误') 
        
        self.ok_code = 0 # 定义成功返回码


    def _post(self, uri, param):
        """
        HTTP POST调用基础方法
        """
        self._refresh_token_if_necessary()

        headers = {
            'Content-Type': CONTENT_FORM_URLENCODED,
            'access_token': self.token
        }

        data = self._encode_param(param)
        response = requests.post(GW_URL + self._process_uri(uri), headers=headers, data=data)
        return self._parse_repsonse(uri, response)

            
    
    def _get(self, uri, param):
        """
        HTTP GET调用基础方法
        """
        self._refresh_token_if_necessary()

        headers = {
            'access_token': self.token
        }

        data = self._encode_param(param)
        response = requests.get("%s%s?%s"%(GW_URL, self._process_uri(uri), data), headers=headers)
        return self._parse_repsonse(uri, response)

    
    def _parse_repsonse(self, uri, response):
        if response.status_code == 200: 
            result = json.loads(response.text)
            if result['code'] == self.ok_code:
                return result['data']
            else:
                raise RuntimeError("failed to call %s: %s" % ( uri, result['msg']))
        else:
            raise RuntimeError("failed to call %s, http stauts code is %d" % (uri, response.status_code))

    
    def _encode_param(self, param):
        if isinstance(param, dict):
            data = parse.urlencode(param)
        elif isinstance(param, str):
            data = param
        else:
            data = parse.urlencode(param.__dict__)
        return data
        

    def _process_uri(self, uri):
        if uri.startswith('/'):
            return uri
        else:
            return '/' + uri
    
    def _refresh_token_if_necessary(self):
        """
        刷新token 如果有必要
        """
        if self.refreshable:
            delta = datetime.now() - self.last_token_time
            if delta.total_seconds() >= self.token_expire_seconds:
                token_data = refresh_token(self.client_secret, self.client_secret, self.refresh_token)
                self.token = token_data[ACCESS_TOKEN]
                self.refresh_token = token_data[REFRESH_TOKEN]
                self.token_expire_seconds = token_data[EXPIRES_IN]
                self.last_token_time = datetime.now()
                



            
        


        



